@@ -1,15 +1,16 @@
 """Define V2 and V3 SimpliSafe systems."""
-import asyncio
+from __future__ import annotations
+
 from dataclasses import dataclass
 from datetime import datetime
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, cast
 
 from simplipy.const import LOGGER
-from simplipy.entity import EntityTypes
+from simplipy.device import DeviceTypes
+from simplipy.device.sensor.v2 import SensorV2
+from simplipy.device.sensor.v3 import SensorV3
 from simplipy.errors import PinError, SimplipyError
-from simplipy.sensor.v2 import SensorV2
-from simplipy.sensor.v3 import SensorV3
 from simplipy.util.dt import utc_from_timestamp
 from simplipy.util.string import convert_to_underscore
 
@@ -30,15 +31,6 @@ MAX_PIN_LENGTH = 4
 RESERVED_PIN_LABELS = {CONF_DURESS_PIN, CONF_MASTER_PIN}
 
 
-def get_entity_type_from_data(entity_data: Dict[str, Any]) -> EntityTypes:
-    """Get the entity type of a raw data payload."""
-    try:
-        return EntityTypes(entity_data["type"])
-    except ValueError:
-        LOGGER.error("Unknown entity type: %s", entity_data["type"])
-        return EntityTypes.unknown
-
-
 @dataclass(frozen=True)
 class SystemNotification:
     """Define a representation of a system notification."""
@@ -49,8 +41,8 @@ class SystemNotification:
     code: str
     timestamp: float
 
-    link: Optional[str] = None
-    link_label: Optional[str] = None
+    link: str | None = None
+    link_label: str | None = None
 
     def __post_init__(self) -> None:
         """Run post-init initialization."""
@@ -74,13 +66,13 @@ class SystemStates(Enum):
     unknown = 99
 
 
-def coerce_state_from_raw_value(value: str) -> SystemStates:
-    """Return a proper state from a string input."""
+def get_device_type_from_data(device_data: dict[str, Any]) -> DeviceTypes:
+    """Get the device type of a raw data payload."""
     try:
-        return SystemStates[convert_to_underscore(value)]
-    except KeyError:
-        LOGGER.error("Unknown system state: %s", value)
-        return SystemStates.unknown
+        return DeviceTypes(device_data["type"])
+    except ValueError:
+        LOGGER.error("Unknown device type: %s", device_data["type"])
+        return DeviceTypes.unknown
 
 
 def guard_from_missing_data(default_value: Any = None) -> Callable:
@@ -104,31 +96,28 @@ def guard_from_missing_data(default_value: Any = None) -> Callable:
     return decorator
 
 
-class System:  # pylint: disable=too-many-instance-attributes,too-many-public-methods
+class System:
     """Define a system.
 
     Note that this class shouldn't be instantiated directly; it will be instantiated as
-    appropriate via :meth:`simplipy.API.get_systems`.
+    appropriate via :meth:`simplipy.API.async_get_systems`.
 
-    :param request: A method to make authenticated API requests.
-    :type request: ``Callable[..., Coroutine]``
-    :param get_subscription_data: A method to get the latest subscription data
-    :type get_subscription_data: ``Callable[..., Coroutine]``
-    :param location_info: A raw data dict representing the system's state and properties.
-    :type location_info: ``dict``
+    :param api: A :meth:`simplipy.API` object
+    :type api: :meth:`simplipy.API`
+    :param sid: A subscription ID
+    :type sid: ``int``
     """
 
-    def __init__(self, api: "API", system_id: int) -> None:
+    def __init__(self, api: "API", sid: int) -> None:
         """Initialize."""
         self._api = api
-        self._system_id = system_id
+        self._sid = sid
 
         # These will get filled in after initial update:
-        self._notifications: List[SystemNotification] = []
+        self._notifications: list[SystemNotification] = []
         self._state = SystemStates.unknown
-        self.entity_data: Dict[str, Dict[str, Any]] = {}
-
-        self.sensors: Dict[str, Union[SensorV2, SensorV3]] = {}
+        self.sensor_data: dict[str, dict[str, Any]] = {}
+        self.sensors: dict[str, SensorV2 | SensorV3] = {}
 
     @property  # type: ignore
     @guard_from_missing_data()
@@ -137,9 +126,7 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
 
         :rtype: ``str``
         """
-        return cast(
-            str, self._api.subscription_data[self._system_id]["location"]["street1"]
-        )
+        return cast(str, self._api.subscription_data[self._sid]["location"]["street1"])
 
     @property  # type: ignore
     @guard_from_missing_data(False)
@@ -150,19 +137,7 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         """
         return cast(
             bool,
-            self._api.subscription_data[self._system_id]["location"]["system"][
-                "isAlarming"
-            ],
-        )
-
-    @property
-    def active(self) -> bool:
-        """Return whether the system is active.
-
-        :rtype: ``bool``
-        """
-        return cast(
-            bool, self._api.subscription_data[self._system_id]["activated"] != 0
+            self._api.subscription_data[self._sid]["location"]["system"]["isAlarming"],
         )
 
     @property  # type: ignore
@@ -174,13 +149,11 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         """
         return cast(
             str,
-            self._api.subscription_data[self._system_id]["location"]["system"][
-                "connType"
-            ],
+            self._api.subscription_data[self._sid]["location"]["system"]["connType"],
         )
 
     @property
-    def notifications(self) -> List[SystemNotification]:
+    def notifications(self) -> list[SystemNotification]:
         """Return the system's current messages/notifications.
 
         :rtype: ``List[:meth:`simplipy.system.SystemNotification`]``
@@ -196,9 +169,7 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         """
         return cast(
             str,
-            self._api.subscription_data[self._system_id]["location"]["system"][
-                "serial"
-            ],
+            self._api.subscription_data[self._sid]["location"]["system"]["serial"],
         )
 
     @property
@@ -216,7 +187,7 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
 
         :rtype: ``int``
         """
-        return self._system_id
+        return self._sid
 
     @property  # type: ignore
     @guard_from_missing_data()
@@ -227,9 +198,7 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         """
         return cast(
             int,
-            self._api.subscription_data[self._system_id]["location"]["system"][
-                "temperature"
-            ],
+            self._api.subscription_data[self._sid]["location"]["system"]["temperature"],
         )
 
     @property  # type: ignore
@@ -241,32 +210,30 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         """
         return cast(
             int,
-            self._api.subscription_data[self._system_id]["location"]["system"][
-                "version"
-            ],
+            self._api.subscription_data[self._sid]["location"]["system"]["version"],
         )
 
-    async def _set_updated_pins(self, pins: Dict[str, Any]) -> None:
-        """Post new PINs."""
-        raise NotImplementedError()
-
-    async def _set_state(self, value: SystemStates) -> None:
+    async def _async_set_state(self, value: SystemStates) -> None:
         """Raise if calling this undefined based method."""
         raise NotImplementedError()
 
-    async def _update_entity_data(self, cached: bool = False) -> None:
-        """Update all entity data."""
+    async def _async_set_updated_pins(self, pins: dict[str, Any]) -> None:
+        """Post new PINs."""
         raise NotImplementedError()
 
-    async def _update_settings_data(self, cached: bool = True) -> None:
+    async def _async_update_device_data(self, cached: bool = False) -> None:
+        """Update all device data."""
+        raise NotImplementedError()
+
+    async def _async_update_settings_data(self, cached: bool = True) -> None:
         """Update all settings data."""
-        pass
+        raise NotImplementedError()
 
-    async def _update_system_data(self) -> None:
-        """Update all system data."""
-        await self._api.update_subscription_data()
+    async def _async_update_subscription_data(self) -> None:
+        """Update subscription data."""
+        await self._api.async_update_subscription_data()
 
-    async def clear_notifications(self) -> None:
+    async def async_clear_notifications(self) -> None:
         """Clear all active notifications.
 
         This will remove the notifications from SimpliSafe's cloud, meaning they will no
@@ -278,13 +245,13 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
             )
             self._notifications = []
 
-    async def generate_entities(self) -> None:
-        """Generate entity objects for this system."""
+    def generate_device_objects(self) -> None:
+        """Generate device objects for this system."""
         raise NotImplementedError()
 
-    async def get_events(
-        self, from_datetime: Optional[datetime] = None, num_events: Optional[int] = None
-    ) -> List[Dict[str, Any]]:
+    async def async_get_events(
+        self, from_datetime: datetime | None = None, num_events: int | None = None
+    ) -> list[dict[str, Any]]:
         """Get events recorded by the base station.
 
         If no parameters are provided, this will return the most recent 50 events.
@@ -307,19 +274,19 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
 
         return cast(List[Dict[str, Any]], events_resp.get("events", []))
 
-    async def get_latest_event(self) -> dict:
+    async def async_get_latest_event(self) -> dict:
         """Get the most recent system event.
 
         :rtype: ``dict``
         """
-        events = await self.get_events(num_events=1)
+        events = await self.async_get_events(num_events=1)
 
         try:
             return events[0]
         except IndexError:
-            raise SimplipyError("SimpliSafe cloud didn't return any events") from None
+            raise SimplipyError("SimpliSafe didn't return any events") from None
 
-    async def get_pins(self, cached: bool = True) -> Dict[str, str]:
+    async def async_get_pins(self, cached: bool = True) -> dict[str, str]:
         """Return all of the set PINs, including master and duress.
 
         The ``cached`` parameter determines whether the SimpliSafe Cloud uses the last
@@ -331,7 +298,7 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         """
         raise NotImplementedError()
 
-    async def remove_pin(self, pin_or_label: str) -> None:
+    async def async_remove_pin(self, pin_or_label: str) -> None:
         """Remove a PIN by its value or label.
 
         :param pin_or_label: The PIN value or label to remove
@@ -340,7 +307,7 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         # Because SimpliSafe's API works by sending the entire payload of PINs, we
         # can't reasonably check a local cache for up-to-date PIN data; so, we fetch the
         # latest each time:
-        latest_pins = await self.get_pins(cached=False)
+        latest_pins = await self.async_get_pins(cached=False)
 
         if pin_or_label in RESERVED_PIN_LABELS:
             raise PinError(f"Refusing to delete reserved PIN: {pin_or_label}")
@@ -352,21 +319,21 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
 
         del latest_pins[label]
 
-        await self._set_updated_pins(latest_pins)
+        await self._async_set_updated_pins(latest_pins)
 
-    async def set_away(self) -> None:
+    async def async_set_away(self) -> None:
         """Set the system in "Away" mode."""
-        await self._set_state(SystemStates.away)
+        await self._async_set_state(SystemStates.away)
 
-    async def set_home(self) -> None:
+    async def async_set_home(self) -> None:
         """Set the system in "Home" mode."""
-        await self._set_state(SystemStates.home)
+        await self._async_set_state(SystemStates.home)
 
-    async def set_off(self) -> None:
+    async def async_set_off(self) -> None:
         """Set the system in "Off" mode."""
-        await self._set_state(SystemStates.off)
+        await self._async_set_state(SystemStates.off)
 
-    async def set_pin(self, label: str, pin: str) -> None:
+    async def async_set_pin(self, label: str, pin: str) -> None:
         """Set a PIN.
 
         :param label: The label to use for the PIN (shown in the SimpliSafe app)
@@ -385,7 +352,7 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         # Because SimpliSafe's API works by sending the entire payload of PINs, we
         # can't reasonably check a local cache for up-to-date PIN data; so, we fetch the
         # latest each time.
-        latest_pins = await self.get_pins(cached=False)
+        latest_pins = await self.async_get_pins(cached=False)
 
         if pin in latest_pins.values():
             raise PinError(f"Refusing to create duplicate PIN: {pin}")
@@ -396,14 +363,14 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
 
         latest_pins[label] = pin
 
-        await self._set_updated_pins(latest_pins)
+        await self._async_set_updated_pins(latest_pins)
 
-    async def update(
+    async def async_update(
         self,
         *,
-        include_system: bool = True,
+        include_subscription: bool = True,
         include_settings: bool = True,
-        include_entities: bool = True,
+        include_devices: bool = True,
         cached: bool = True,
     ) -> None:
         """Get the latest system data.
@@ -411,29 +378,23 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
         The ``cached`` parameter determines whether the SimpliSafe Cloud uses the last
         known values retrieved from the base station (``True``) or retrieves new data.
 
-        :param include_system: Whether system state/properties should be updated
-        :type include_system: ``bool``
+        :param include_subscription: Whether system state/properties should be updated
+        :type include_subscription: ``bool``
         :param include_settings: Whether system settings (like PINs) should be updated
         :type include_settings: ``bool``
-        :param include_entities: whether sensors/locks/etc. should be updated
-        :type include_entities: ``bool``
+        :param include_devices: whether sensors/locks/etc. should be updated
+        :type include_devices: ``bool``
         :param cached: Whether to used cached data.
         :type cached: ``bool``
         """
-        tasks = []
-
-        if include_system:
-            tasks.append(self._update_system_data())
+        if include_subscription:
+            await self._async_update_subscription_data()
         if include_settings:
-            tasks.append(self._update_settings_data(cached))
-
-        await asyncio.gather(*tasks)
-
-        # We await entity updates after the task pool since including it can cause
-        # HTTP 409s if that update occurs out of sequence:
-        if include_entities:
-            await self._update_entity_data(cached)
+            await self._async_update_settings_data(cached)
+        if include_devices:
+            await self._async_update_device_data(cached)
 
+        # Create notifications:
         self._notifications = [
             SystemNotification(
                 raw_message["id"],
@@ -444,13 +405,18 @@ class System:  # pylint: disable=too-many-instance-attributes,too-many-public-me
                 link=raw_message["link"],
                 link_label=raw_message["linkLabel"],
             )
-            for raw_message in self._api.subscription_data[self._system_id]["location"][
+            for raw_message in self._api.subscription_data[self._sid]["location"][
                 "system"
             ].get("messages", [])
         ]
 
-        self._state = coerce_state_from_raw_value(
-            self._api.subscription_data[self._system_id]["location"]["system"].get(
-                "alarmState"
-            )
+        # Set the current state:
+        raw_state = self._api.subscription_data[self._sid]["location"]["system"].get(
+            "alarmState"
         )
+
+        try:
+            self._state = SystemStates[convert_to_underscore(raw_state)]
+        except KeyError:
+            LOGGER.error("Unknown raw system state: %s", raw_state)
+            self._state = SystemStates.unknown
